#!/usr/bin/python
import debinterface.interfaces
import commands
import os
import glob


INTERFACES_FILEPATH = '/etc/network/interfaces'
MANAGEMENT_MAC_FILEPATH = '/etc/network/management-mac'
DHCPD_CONF_FILEPATH = '/etc/dhcp/dhcpd.conf'
DHCP_DEFAULT_FILEPATH = '/etc/default/isc-dhcp-server'

NETMASK = '255.255.255.0'
IPADDRESS_TEMPLATE = '192.168.%s4.1'
MTU = '6000'

DHCP_SUBNET_TEMPLATE = \
    'subnet 192.168.%s4.0 netmask 255.255.255.0 { \n \
      pool { \n \
        max-lease-time 43200; \n \
        range 192.168.%s4.2 192.168.%s4.20; \n \
      } \n\
    }'


if_list = glob.glob('/sys/class/net/*/address')
name_to_mac_map = {}

for iface_filepath in if_list:
    with open(iface_filepath, 'r') as iface_file:
        name_to_mac_map[iface_filepath.split('/')[-2]] = iface_file.readlines()[0].strip()

iface_conf = debinterface.interfaces.Interfaces(interfaces_path=INTERFACES_FILEPATH)

stdout = commands.getoutput("ls /sys/class/net/ | grep eth")
# adapter_list = stdout.split('\n')
adapter_list = name_to_mac_map.keys()

with open(MANAGEMENT_MAC_FILEPATH, 'r') as mgmt_mac_file:
    management_mac = mgmt_mac_file.readlines()[0].strip()

# exclude management adapter
for adapter in adapter_list:
    if name_to_mac_map[adapter] == management_mac:
        adapter_list.remove(adapter)

# exclude loopback adapter
adapter_list.remove('lo')

dhcp_config = []
ip_octet = 1
for adapter in adapter_list:
    new_adapter = debinterface.interfaces.NetworkAdapter(adapter)
    new_adapter.setAuto(True)
    new_adapter.setAddrFam('inet')
    new_adapter.setAddressSource('static')
    new_adapter.setAddress(IPADDRESS_TEMPLATE % ip_octet)
    new_adapter.setNetmask(NETMASK)

    adapter_options = new_adapter.export()
    adapter_options['mtu'] = MTU

    iface_conf.addAdapter(adapter_options, 0)

    dhcp_subnet = DHCP_SUBNET_TEMPLATE % (ip_octet, ip_octet, ip_octet)
    dhcp_config.append(dhcp_subnet)

    ip_octet += 1

iface_conf.writeInterfaces()

dhcp_config_contents = '\n'.join(dhcp_config)
os.system('echo \"%s\" >> %s' % (dhcp_config_contents, DHCPD_CONF_FILEPATH))

adapter_list_string = ' '.join(adapter_list)

with open('/etc/debian_version', 'r') as debver:
    version = int(float(debver.readlines()[0].strip()))

if version == 9:
    # for Debian Stretch
    sed_command = '''sed -i -e 's/INTERFACESv4=""/INTERFACESv4="%s"/' %s''' % (adapter_list_string, DHCP_DEFAULT_FILEPATH)

elif version == 8:
    # for Debian Jessie
    sed_command = '''sed -i -e 's/INTERFACES=""/INTERFACES="%s"/' %s''' % (adapter_list_string, DHCP_DEFAULT_FILEPATH)

else:
    sed_command = "false"

os.system(sed_command)
