import random


class _LrColorContext:
    # http://www.stm.dp.ua/web-design/color-html.php
    _COLORS = [(255, 248, 220),  # Cornsilk
               (255, 228, 181),  # Moccasin
               (255, 245, 238),  # Seashell
               (240, 255, 240),  # Honeydew
               (230, 230, 250),  # lavender
               (240, 255, 255),  # Azure
               (255, 228, 225),  # MistyRose
               (152, 251, 152),  # PaleGreen
               (255, 215, 0),  # Gold
               (188, 143, 143),  # RosyBrown
               (245, 245, 220),  # Beige
               (245, 222, 179),  # Wheat
               (193, 255, 193),  # DarkSeaGreen1
               (84, 255, 159),  # SeaGreen1
               ]

    def __init__(self, rm_delayed_call):
        """
        :type rm_delayed_call: twisted.internet.interfaces.IDelayedCall
        """
        self.color = random.choice(_LrColorContext._COLORS)
        self.rm_delayed_call = rm_delayed_call


class LrColorToutStore:
    def __init__(self, twisted_reactor, lr_color_store_tout):
        """
        :type twisted_reactor: twisted.internet.interfaces.IReactorTime
        :type lr_color_store_tout: int
        """
        self._twisted_reactor = twisted_reactor
        self._lr_color_store_tout = lr_color_store_tout
        self._lr_oid_to_context_map = {}

    def _handle_lr_oid_tout(self, lr_oid):
        self._lr_oid_to_context_map.pop(lr_oid, None)

    def get_lr_oid_color(self, lr_oid):
        """
        :type lr_oid: int
        :rtype: (int, int, int)
        """
        context = self._lr_oid_to_context_map.get(lr_oid, None)

        if context:
            context.rm_delayed_call.reset(self._lr_color_store_tout)

        else:
            # Schedule to remove entry after _lr_color_store_tout since the last update.
            rm_delayed_call = self._twisted_reactor.callLater(
                self._lr_color_store_tout, self._handle_lr_oid_tout, lr_oid)

            context = _LrColorContext(rm_delayed_call)
            self._lr_oid_to_context_map[lr_oid] = context

        return context.color
