from edge.image.image import ImRgba
from PIL import Image


from edge.queue import JsonFileQueue, ImageFileQueue
from _args import EventArgs, ImStoreArgs


class SceneEventConsumer:
    _STATUS_SUCCESS = 1
    _STATUS_FAILURE = 2

    def __init__(self, twisted_reactor, codec, json_file_queue_list, image_file_queue):
        """
        :type codec: str
        :type json_file_queue_list: list[JsonFileQueue]
        :type image_file_queue: ImageFileQueue
        """
        self._twisted_reactor = twisted_reactor
        self._target_codec = codec
        self._json_file_queue_list = json_file_queue_list
        self._image_file_queue = image_file_queue

    def consume_scene(self, scene_ts, blob):
        """
        :type scene_ts: int
        :type blob: ImRgba
        """
        w, h = blob.wh
        with Image.frombuffer(
                'RGBA', (w, h + blob.dh), blob.buffer,
                'raw', 'RGBA', blob.stride, 1) as pil_img:

            image_ref = self._image_file_queue.add_image(scene_ts, pil_img)

            event_args_jnode = {
                'Ts': image_ref.ts,
                'Status': SceneEventConsumer._STATUS_SUCCESS,  # todo
                'SizeWh': pil_img.size,
                'Url': image_ref.url,
                'Codec': image_ref.codec
            }

            map(lambda q: q.add_jnode(event_args_jnode), self._json_file_queue_list)

    @staticmethod
    def create(twisted_reactor, scene_args, store_args):
        """
        :type twisted_reactor: twisted.internet.interfaces.IReactorTime
        :type scene_args: EventArgs
        :type store_args: ImStoreArgs
        """
        json_file_queue_list = map(
            lambda queue_args: JsonFileQueue(queue_args), scene_args.json_queue_args_list)

        image_file_queue = ImageFileQueue(
            twisted_reactor, store_args.codec,
            store_args.image_queue_args.endpoint,
            store_args.image_queue_args.min_file_count,
            store_args.image_queue_args.max_file_count,
            scene_args.log_level)

        return SceneEventConsumer(twisted_reactor, store_args.codec, json_file_queue_list, image_file_queue)
