import time
from os import rename
import ujson

from edge import eventlog


from ._args import StatsServiceArgs


class IStatService:

    def __init__(self):
        pass

    @property
    def wh(self):
        """
        :rtype: WhCounters
        """
        raise NotImplementedError


def init_service(twisted_reactor, args):
    """
    :type twisted_reactor: twisted.internet.interfaces.IReactorTime
    :type args: StatsServiceArgs
    :rtype: IStatService
    """
    if args.stat_tout_sec:
        stat_service = _StatService(twisted_reactor, args)
    else:
        stat_service = _StubStatService

    return stat_service


# noinspection PyUnusedLocal
def fini_service(service):
    """
    :type service: IStatService | _StatService
    """
    pass


class WhCounters:
    def __init__(self):
        self.w = 0
        self.h = 0
        self.dh = 0

    def im_size(self, w, h, dh):
        """
        :type w: int
        :type h: int
        :type dh: int
        """
        self.w = w
        self.h = h
        self.dh = dh


class _StubStatService(IStatService):
    def __init__(self):
        IStatService.__init__(self)

        self._wh = WhCounters()

    @property
    def wh(self):
        """
        :rtype: WhCounters
        """
        return self._wh


class _StatService(IStatService):
    def __init__(self, twisted_reactor, args):
        """
        :type twisted_reactor: twisted.internet.interfaces.IReactorTime
        :type args: StatsServiceArgs
        """
        IStatService.__init__(self)

        self._wh = WhCounters()

        self._stat_abspath = args.stat_filepath
        self._stat_tmp_abspath = args.stat_filepath + '.tmp'
        self._stat_tout_sec = args.stat_tout_sec

        if args.stat_tout_sec > 0:
            twisted_reactor.callLater(args.stat_tout_sec, self._log_counters, twisted_reactor)

    @property
    def wh(self):
        """
        :rtype: WhCounters
        """
        return self._wh

    def _log_counters(self, twisted_reactor):
        self._report_counters()
        twisted_reactor.callLater(self._stat_tout_sec, self._log_counters, twisted_reactor)

    def _report_counters(self):
        sys_utc_time = time.time()

        stat = {
            "stat": {
                "mtime": [time.strftime('%d.%m.%Y %H:%M:%S', time.gmtime(sys_utc_time)), int(round(sys_utc_time))],
                "tout-sec": self._stat_tout_sec
            },
            "stream": {
                "wh": [self.wh.w, self.wh.h + self.wh.dh],
                "dh": self.wh.dh
            }
        }

        # noinspection PyBroadException
        try:
            with open(self._stat_tmp_abspath, 'w') as json_file:
                # Use ensure_ascii=false to output UTF-8.
                ujson.dump(stat, json_file, ensure_ascii=False)

            rename(self._stat_tmp_abspath, self._stat_abspath)

        except Exception:
            eventlog.log_x()
