#! /usr/bin/env python
# -*- coding: utf-8 -*-
# (k) made-on-the-knee-of /dragon, 2019

import os,sys,ujson
import errno, traceback
from datetime import datetime, timedelta
from time import time
import re
import base64
import psycopg2
import psycopg2.extras
import psycopg2.extensions
psycopg2.extensions.register_type(psycopg2.extensions.UNICODE)
psycopg2.extensions.register_type(psycopg2.extensions.UNICODEARRAY)

from edge import eventlog

from pyblobstore import bss, vss

from PIL import Image
import io


conf = {
    'dbhost': '127.0.0.1',
    'dbport': 5433,
    'dbname': 'autocode',
    'dbuser': 'autocode',
    'dbpassword': 'autocode',
    'dist': 40,
    'z1': 20,
    'z2': 12,
    'rlen': 1.1
}

flagref = {
    0: "Нет группировки",
    1: "Совпадение",
    3: "Перестроение",
    5: "Несовпадение номеров",
    7: "Несовпадение номеров и перестроение",
    8: "Не определен передний номер",
    16:"Не определен задний номер"
}


if __name__ == '__main__':
    try:
        eventlog.open_log('acs_report', 'debug')
        eventlog.log_i('startup...')

        if len(sys.argv) < 5:
            eventlog.log_e("invalid arguments: "+str(sys.argv))
            sys.exit(1)         
        
        _stime = datetime(year=int(sys.argv[1]),month=int(sys.argv[2]),day=int(sys.argv[3]),hour=int(sys.argv[4]))
        
        try:
            dbconn = psycopg2.connect("host=%s port=%d dbname=%s user=%s password=%s" % (
                 conf['dbhost']
                ,conf['dbport']
                ,conf['dbname']
                ,conf['dbuser']
                ,conf['dbpassword']
            ))
        except:
            eventlog.log_e("Error db setup")
            sys.exit(1)
        try:
            bs_args = bss.BsServiceArgs(log_level='debug')
            bs_service = bss.init_service(bs_args)
            bs_channel = 0
            def _detail(dbconn, guid):
                with dbconn:
                    with dbconn.cursor(cursor_factory=psycopg2.extras.RealDictCursor) as dbc:
                        dbc.execute("""
                            select 
                                ei.guid,
                                ei.ctime,
                                to_char(ei.ctime,'DD-MM-YYYY') as date,
                                to_char(ei.ctime,'HH24:MI:SS.MS') as time,
                                ei.lane,
                                ei.dir,
                                ei.plate,
                                ei.country,
                                ei.make,ei.model,ei.category,
                                ed.dist,
                                eo.origin
                            from 
                                eventindex ei 
                                join eventdist ed using(id) 
                                join collection_event_origin eo on eo.id = ei.origin  
                            where ei.guid = '{0}';
                        """.format(guid),1)
                        rv = dbc.fetchone()
                        return rv
            def _image(r):
                _fn = r['dist']['selected']['vehicle'] if 'vehicle' in r['dist']['selected'] else r['dist']['selected']['scene'] if 'scene' in r['dist']['selected'] else None
                if _fn:
                    _a = re.split(r'[_\.\n]',_fn)
                    _ch = _a[0]
                    _ts = int(_a[1])
                    blob = bs_service.load(_ch, _ts)
                    if blob.file_data:
                        im = Image.open(io.BytesIO(blob.file_data))
                        im.thumbnail([480,480],Image.ANTIALIAS)
                        bff = io.BytesIO()
                        im.save(bff,format="JPEG") 
                        return "data:image/jpeg;base64,"+base64.b64encode(bff.getvalue())
                return None
            def _block(r):
                return """
                <img src="{image}"/>
                <div class="block bfull"><div class="plate">{plate}</div></div>
                <div class="block bfull">{mmc}</div>
                <div class="block bfull">
                <div class="block bhalf">
                {date}<br/>
                {time}
                </div><div class="block bhalf">
                {place}<br/>
                {lane}
                </div></div>""".format(
                    image=_image(r)
                    ,plate=(r['plate'].encode('utf-8') if r['plate'] and not r['plate']=="" else "&nbsp;")
                    ,date=r['date']
                    ,time=r['time']
                    ,place=r['origin']['location']['place'].encode('utf-8')
                    ,lane=r['lane'].encode('utf-8')
                    ,mmc=r['make']+' '+r['model']+', '+r['category'] if r['make'] else '&nbsp;'
                )
            def _led(tag,yes,draw=False):
                return("""<div style="display: inline-block; min-width: 5em; margin: 2px; text-align: center; background-color: %s; color: %s">%s</div>"""%(
                    'green' if yes else 'red',
                    'white' if yes else 'yellow',
                    tag if (yes == draw) else "&nbsp;"
                ))
            with dbconn:
                with dbconn.cursor(cursor_factory=psycopg2.extras.RealDictCursor) as dbc:
                    dbc.execute("""
                        select 
                            guid, prev, flag
                        from acs.crossing
                        where 
                            ctime >= %s
                            and ctime <= %s
                            and flag > 0
                            and tozone = 0
                        order by ctime    
                     """,(_stime, _stime+timedelta(hours=1),))
                    rv = dbc.fetchall() 
                    print("""
        <style type="text/css" scoped>
        .block{ display: block; position: relative; align: center; } .bfull{ width: 100%; } .bhalf{ display: inline-block; width: 50%; } .plate{ display: inline-block; font-weight: 700; font-family: sans-serif; color: #000; background-color: #fff; font-size: 2rem; border: 2px solid #000; border-radius: 5px; margin: 10px; width: 14rem!important; }
        </style>
        <table border="1" align="center" width="95%" cellspacing="0" cellpadding="2">
                    """)
                    __n = 0
                    __ok = 0
                    __nmmr = 0
                    __fmmr = 0
                    __fplt = 0
                    __nplt = 0
                    for r in rv:
                        _rear  = _detail(dbconn,r['guid'])
                        _front = _detail(dbconn,r['prev'])
                        __n += 1
                        if _front['make'] is None and _rear['make'] is None: 
                            __nmmr += 1
                        elif _front['make'] != _rear['make'] or _front['model'] != _rear['model']:
                            __fmmr += 1
                        else:
                            __ok += 1
                        if r['flag']&16 > 0:
                            __fplt += 1
                        if r['flag'] > 32:
                            __nplt += 1
                        print("""<tr><td align="center" valign="bottom">""")
                        print(_block(_front))
                        print("""</td><td align="center" valign="bottom">""")
                        print(_block(_rear))
                        print("""</td></tr>""")
                        print("""<tr><td colspan="2" align="center"><big>""")
                        print(_led("!GOT",r['flag']&1>0,True))
                        print(_led("!CLN",r['flag']&2==0))
                        print(_led("!FPN",r['flag']&4==0))
                        print(_led("!MMR",not(r['flag']&8>0 or not _front['make'] or _front['make'] != _rear['make'] or _front['model'] != _rear['model'] )))
                        print(_led("!PLT",r['flag']&16==0))
                        print(_led("!NFR",r['flag']&32==0))
                        print(_led("!NRR",r['flag']&64==0))
                        print("""</big></td></tr>""")
                        
                    print("""
        </table>
        <br/>
        total: {0}<br/>
        mmrOK: {1}<br/>
        mmrFU: {2}<br/>
        mmrNO: {3}<br/>
        pltAB: {4}<br/>
        pltNO: {5}<br/>
                    """.format(__n,__ok,__fmmr,__nmmr,__fplt,__nplt))

        finally:
            bss.fini_service(bs_service)

    except SystemExit:
        pass
    except BaseException, e:
        eventlog.log_e("CROSS ERROR: %s with %s"%(str(e),traceback.format_exc()))
    finally:
        eventlog.log_i('finish...')
        eventlog.close_log()

