
from edge import eventlog
from edge.image.types import Blob, FileBlob, MemBlob, StructBlobMeta

from edge.com import bs


class BsServiceArgs:
    def __init__(self, log_level):
        """
        :type log_level: str
        """
        self.log_level = log_level


class IBsService:
    def __init__(self):
        pass

    def save(self, chan, blob):
        """
        :type chan: str
        :type blob: Blob
        :raise bs.BsError:
        """
        raise NotImplementedError


def init_service(args):
    """
    :type args: BsServiceArgs
    :rtype: IBsService
    """
    return _BsService(args)


def fini_service(service):
    """
    :type service: IBsService | _BsService
    """
    service.destroy()


class _BsService(IBsService):
    def __init__(self, bs_conf):
        """
        :type bs_conf: BsServiceArgs
        """
        IBsService.__init__(self)
        counters = bs.StubBsCounters()
        self._session_args = counters, 65535, bs.LOG_LEVEL_NAME_TO_CODE_MAP.get(bs_conf.log_level, 'info')
        self._session = bs.Session(counters=counters, closed=True)

    def destroy(self):
        self._session.close()

    @bs.reopen_session
    def save(self, chan, blob):
        """
        :type chan: str
        :type blob: MemBlob | FileBlob

        :raise bs.BsError:
        :raise bs.BsSessionError:
        """
        try:
            blob_ts = blob.min_ts / 1000
            if blob.file_path is None:
                if isinstance(blob.meta, StructBlobMeta):
                    ident = self._session.save_blob(chan, blob_ts, blob.file_data, blob.meta.struct)
                else:
                    ident = self._session.save_blob(chan, blob_ts, blob.file_data, blob.file_meta)
            else:
                ident = self._session.save_blob2(chan, blob_ts, blob.file_path, blob.file_meta)

        except bs.BsError:
            eventlog.log_e('fail: save <chan={}, profile={}>'.format(str(chan), blob.profile))
            raise

        blob.name = '%s_%s.%s' % (chan, ident, blob.file_ext)


    @bs.reopen_session
    def load(self, chan, ts):
        """
        :type chan: str
        :type ts: long
        :param ts: timestamp in us
        :raise bs.BsError:
        :raise bs.BsSessionError:
        """
        try:
            blob_ts = ts
            
            _data, _data_size, _meta, _meta_size = self._session.load_blob(chan, blob_ts)

            return MemBlob('', ts, str(_data), meta = _meta if _meta_size > 0 else None, file_ext='')

        except bs.BsError:
            eventlog.log_e('fail: load <chan={}, profile={}>'.format(str(chan), "?"))
            raise
