#! /usr/bin/python2
# -*- coding: utf-8 -*-
# (k) made-on-the-knee-of /dragon, 2019

import os,sys
import errno, traceback

import ujson
from datetime import datetime, timedelta
from time import time
import re, uuid

from io import BytesIO
from PIL import Image

import psycopg2
import psycopg2.extras
import psycopg2.extensions
psycopg2.extensions.register_type(psycopg2.extensions.UNICODE)
psycopg2.extensions.register_type(psycopg2.extensions.UNICODEARRAY)

from edge import eventlog
from pyblobstore import bss, vss

reload(sys)
sys.setdefaultencoding('utf8')

pipes = {}
conf = {}

if len(sys.argv) < 2:
    eventlog.log_e("REDLIGHT: Usage: redlight.py <config.json>")
    sys.exit(1)
    
try:
    with open(sys.argv[1],'rt') as f:
        conf = ujson.load(f)
except:
    eventlog.log_e("REDLIGHT: Can`t load config: " + sys.argv[1])
    sys.exit(1)

def _avgspeed(_dbconn,_stime):
    with _dbconn:
        with _dbconn.cursor(cursor_factory=psycopg2.extras.RealDictCursor) as dbc:
            dbc.execute("""
                select extract(epoch from avg(dt)) as tm
                from (
                    select c2.plate, c2.ctime as t1, c1.ctime as t2 , c1.ctime - c2.ctime as dt 
                    from 
                        acs.crossing c1 
                        join acs.crossing c2 
                        on c1.tozone = 0 
                            and c2.fromzone = 0 
                            and c1.ctime < %s
                            and c1.ctime > %s - interval '1s' * %s
                            and c2.ctime < c1.ctime 
                            and c2.ctime > c1.ctime - interval '1s' * %s 
                            and c2.plate = c1.plate 
                     order by t1 desc) q;            
            """, (datetime.utcfromtimestamp(_stime),datetime.utcfromtimestamp(_stime),conf['maxdepth'],conf['ainterval'],))
            rv = dbc.fetchone()
            return (conf['dist']+conf['z1']/2.0+conf['z2']/2.0)/rv['tm'] if rv and 'tm' in rv and rv['tm'] else None

def _catchimage(service, ts, size=0):
    rts = int(time() * 1000000)
    image = vss.ImRgba()
    eventlog.log_i("CI: (%s) -> %s"%(str(service._image_pool._conf),ts))
    service.export_image_rgba2(image, [vss.VsImageInfo(ts)])
    (w,h) = image.wh
    if w < 8:
        return None,None,None,None
    if size > 0:
        (w,h) = ( int(x*w/size) if w>h else int(x*h/size) for x in (w,h)) 
        imj = vss.ImHelpers.mk_resize_jpeg(image,w,h)
    else:
        imj = vss.ImHelpers.mk_jpeg2(image)
    return imj, w, h, len(imj)    

def _saveblob(service, channel, ts, data, profile, ext):
    blob = bss.MemBlob(profile, ts, data, file_ext=ext)
    service.save(str(channel),blob)
    return blob.name

def _loadblob(service, channel, ts):
    blob = service.load(str(channel),ts)
    return blob.file_data, blob.meta

def _getpipes(dbconn):
    try:
        p = {}
        with open("/etc/edge/setupstore/admin/actions/pipeline.json","rt") as f:
            a = ujson.load(f)
            _p = {x['uuid']:int(x['id']) for x in a} if a else None
            if _p:
                with dbconn:
                    with dbconn.cursor(cursor_factory=psycopg2.extras.RealDictCursor) as dbc:
                        dbc.execute("select * from collection_event_origin")
                        for r in dbc:
                            if r['id'] in _p:
                                p[r['id']] = {
                                    "guid": r['id'],
                                    "pipe": _p[r['id']],
                                    "origin": r['origin'],
                                    "param": r['param']
                                }
        return p                                
    except:
        return None

def _imresize(img, size):
    im = Image.open(BytesIO(img))
    im.thumbnail([size,size],Image.ANTIALIAS)
    w,h = im.size
    bff = BytesIO()
    im.save(bff,format="JPEG") 
    val = bff.getvalue()
    return val, w, h, len(val)

def _imsplash(size, color):
    im = Image.new('RGBA',size,color)
    bff = BytesIO()
    im.save(bff,format="JPEG")
    w,h = im.size
    val = bff.getvalue()
    return val, w, h, len(val)

def _timestamp(d):
    return (d.replace(tzinfo=None)- datetime.utcfromtimestamp(0)).total_seconds()

if __name__ == '__main__':
    bservice = None
    try:
        eventlog.open_log('acs_cross', 'debug')
        eventlog.log_i('startup...')
        
        try:
            dbconn = psycopg2.connect("host=%s port=%d dbname=%s user=%s password=%s" % (
                 conf['dbhost']
                ,conf['dbport']
                ,conf['dbname']
                ,conf['dbuser']
                ,conf['dbpassword']
            ))
            dbconn.autocommit=True
        except:
            eventlog.log_e("REDLIGHT: Error opening database connection!")
            sys.exit(1)
        
        pipes = _getpipes(dbconn)
        if not pipes:
            eventlog.log_e("REDLIGHT: Error getting pipelines!")
            sys.exit(1)
        
        preorg  = next(iter(filter(lambda x: x.get('pipe') == conf['preterminal'], pipes.values())),None)
        postorg = next(iter(filter(lambda x: x.get('pipe') == conf['postterminal'], pipes.values())),None)
        destorg = next(iter(filter(lambda x: x.get('pipe') == conf['destterminal'], pipes.values())),None)
        
        if not preorg or not postorg or not destorg:
            eventlog.log_e("REDLIGHT: Error terminal determination!")
            sys.exit(1)
        
        _bsargs = bss.BsServiceArgs(log_level='debug')
        bservice = bss.init_service(_bsargs)
        
        ntime = time()
        ntime_d = datetime.utcfromtimestamp(ntime)
        ntime_s = ntime_d.isoformat()

        with dbconn:
            with dbconn.cursor(cursor_factory=psycopg2.extras.RealDictCursor) as dbc:
                dbc.execute("""
                    do language plpgsql $$
                        declare
                            _rlp timestamptz;
                            _mtr interval = interval '%s';
                            _opt text = '%s';
                        begin
                            _rlp := _o(_opt);
                            if _rlp isnull then
                                perform putopt(_opt,(now()-_mtr)::text,'','interval','edge',0);
                            elsif _rlp < now() - _mtr then
                                perform setopt(_opt,(now()-_mtr)::text);
                            end if;    
                        end;
                    $$;    
                """ % (conf['maxdepth'],conf['timeopt']))
            dbconn.commit()
            with dbconn.cursor(cursor_factory=psycopg2.extras.RealDictCursor) as dbc:
                dbc.execute("""
                        select
                            e1.ctime as t1,
                            e1.scrollid as s1,
                            e1.guid as  g1,
                            e1.plate as p1,
                            d1.dist as b1,
                            e2.ctime as t2,
                            e2.scrollid as s2,
                            e2.guid as g2,
                            e2.plate as p2,
                            d2.dist as b2
                        from
                        eventindex e1, eventindex e2,
                        eventdist d1, eventdist d2
                        where
                            e1.doctype = 'red-light' and e2.doctype = 'red-light' and
                            e1.origin = %s and
                            e2.origin = %s and
                            e1.ctime > _o(%s)::timestamptz and
                            e1.ctime < now() - interval '1 sec' * %s and
                            abs(extract(epoch from (e2.ctime - e1.ctime))) < %s and
                            e2.plate = e1.plate and
                            d1.id = e1.id and
                            d2.id = e2.id
                        order by t1
                        limit %s
                        ;
                """,(preorg['guid'],postorg['guid'],conf['timeopt'],conf['marginsec'],conf['gapsec'],conf['blocksize'],))
                rv = dbc.fetchall()
                if not rv:
                    eventlog.log_i("REDLIGHT pass2: nothing to do")
                for r in rv:
                    eventlog.log_i("REDLIGHT doing %s -- %s" % (r['t1'],r['g1']))
                    _d1 = next(iter(filter(lambda x: x.get('altref') == 'args.json', r['b1']['list'])),None)
                    if _d1:
                        _a = _d1['name'].replace('.json','').split('_')
                        if _a and len(_a) > 1:
                            _data, _meta = _loadblob(bservice,int(_a[0]),int(_a[1]))        
                            args = ujson.loads(_data)
                            _d2 = next(iter(filter(lambda x: x.get('altref') == 'args.json', r['b2']['list'])),None)
                            if _d2:
                                _a = _d2['name'].replace('.json','').split('_')
                                if _a and len(_a) > 1:
                                    _data, _meta = _loadblob(bservice,int(_a[0]),int(_a[1]))        
                                    args2 = ujson.loads(_data)
                                    _org = destorg['origin'] if 'origin' in destorg and destorg['origin'] else {
                                            "uuid": destorg['guid'],
                                            "location": {
                                                "place": destorg['param']['public']['name'] if 
                                                    'param' in destorg and
                                                    'public' in destorg['param'] and
                                                    'name' in destorg['param']['public']
                                                    else ""
                                            }
                                        }
                                    args['origin'] = _org
                                    _uid = str(uuid.uuid1())
                                    args['uuid'] = _uid
                                    _p1 = args['evidence']['measurements']['waypoints']['post-stopbar']
                                    _p2 = args2['evidence']['measurements']['waypoints']['post-stopbar']
                                    _i2 = [y['name'] for x in [z['images'].values() for z in _p2] for y in x]
                                    _b1 = r['b1']
                                    _b2 = r['b2']
                                    dist = _b1
                                    _l1 = filter(lambda x: x['name'] not in _i2, _b1['list'])
                                    _l2 = filter(lambda x: x['name'] in _i2, _b2['list'])
                                    dist['list'] = filter(lambda x: x.get('altref') <> 'args.json', _l1 + _l2)
                                    args['evidence']['measurements']['waypoints']['post-stopbar'] = _p2
                                    its = int((time()-1) * 1000000) 
                                    aname = _saveblob(bservice,conf['argstore'],its,ujson.dumps(args),'args','json')
                                    dist['list'][:0] = [{
                                        "ref": aname,
                                        "name": aname,
                                        "altref": "args.json"
                                    }]
                                    dist['selected']['args'] = aname
                                    evt = {
                                        "event": args,
                                        "dist": dist,
                                        "flag": 0
                                    }
                                    dbc.execute("select * from api_event_add(%s)",(ujson.dumps(evt),))
                                    rv = dbc.fetchone()
                                    dbconn.commit()
                                    if rv:
                                        if rv['errcode'] == 0:
                                            eventlog.log_i("REDLIGHT: inserted: "+str(_uid))
                                        else:
                                            eventlog.log_e(">> RL: invalid insert result code")
                                    else:
                                        eventlog.log_e(">> RL: invalid insert rep>")
                                else:
                                    eventlog.log_e(">> RL: D2 invalid name")
                            else:
                                eventlog.log_e(">> RL: D2 no args")
                        else:
                            eventlog.log_e(">> RL: D1 invalid name")
                    else:
                        eventlog.log_e(">> RL: D1 no args")
                    dbc.execute("""
                        select setopt(%s,%s)
                    """,(conf['timeopt'],str(r['t1']),))                                                 
            dbconn.commit()
    except SystemExit:
        pass
    except Exception, e:
        eventlog.log_e("REDLIGHT ERROR: %s with %s"%(str(e),traceback.format_exc()))
    finally:
        if bservice:
            bss.fini_service(bservice)
        eventlog.log_i('finish...')
        eventlog.close_log()


