import datetime
from edge import eventlog
from edge.types import JnodeError

from ..types import ScheduleArgs, VideoArgs, BaseRuleArgs
from ....model.jnode.types import JsonTicket


class JsonScheduleArgs(ScheduleArgs):
    def __init__(self, *args, **kwargs):
        ScheduleArgs.__init__(self, *args, **kwargs)

    @staticmethod
    def deserialize(schedule_jnode):
        """
        :type schedule_jnode: dict
        :rtype: ScheduleArgs
        :raise JnodeError:
        """
        # deserialize #/days
        days_jnode = schedule_jnode.get('days')
        if type(days_jnode) != dict or len(days_jnode) != 7:
            eventlog.log_e_param('#/days')
            raise JnodeError

        def deserialize_day(day_a2):
            """
            :type day_a2: str
            :rtype: bool
            """
            turn_on = days_jnode.get(day_a2)
            if bool != type(turn_on):
                eventlog.log_e_param('#/days/%s' % day_a2)

            return turn_on

        days = map(deserialize_day, ['mo', 'tu', 'we', 'th', 'fr', 'sa', 'su'])

        # deserialize #/min-time
        min_time_jnode = schedule_jnode.get('min-time')
        if not type(min_time_jnode) in [list, tuple] or len(min_time_jnode) != 2:
            eventlog.log_e_param('#/min-time')
            raise JnodeError

        hh, mm = min_time_jnode
        if not (int == type(hh) and 0 <= hh <= 23):
            eventlog.log_e_param('#/min-time/0')
            raise JnodeError

        if not (int == type(mm) and 0 <= mm <= 59):
            eventlog.log_e_param('#/min-time/1')
            raise JnodeError

        min_time = datetime.time(hh, mm)

        # deserialize #/max-time
        max_time_jnode = schedule_jnode['max-time']
        if not type(max_time_jnode) in [list, tuple] or len(max_time_jnode) != 2:
            eventlog.log_e_param('#/max-time')
            raise JnodeError

        hh, mm = max_time_jnode
        if not (int == type(hh) and 0 <= hh <= 23):
            eventlog.log_e_param('#/max-time/0')
            raise JnodeError

        if not (int == type(mm) and 0 <= mm <= 59):
            eventlog.log_e_param('#/max-time/1')
            raise JnodeError

        max_time = datetime.time(hh, mm)
        return ScheduleArgs(min_time, max_time, days)


class JsonBaseRuleArgs(BaseRuleArgs):
    def __init__(self, *args, **kwargs):
        BaseRuleArgs.__init__(*args, **kwargs)

    @staticmethod
    def deserialize(rule_jnode):
        """
        :type rule_jnode: dict
        :rtype: BaseRuleArgs
        :raise JnodeError:
        """
        if type(rule_jnode) != dict:
            eventlog.log_e_param('#/')
            raise JnodeError

        # deserialize #/name
        name = rule_jnode.get('name')
        if not (type(name) in [str, unicode]) or len(name) > 64:
            eventlog.log_e_param('#/name')
            raise JnodeError

        # deserialize #/enabled
        enabled = rule_jnode.get('enabled')
        if bool != type(enabled):
            eventlog.log_e_param('#/enabled')
            raise JnodeError

        # deserialize #/schedule
        schedule = None
        schedule_jnode = rule_jnode.get('schedule')
        if schedule_jnode is not None:
            if dict != type(schedule_jnode):
                eventlog.log_e_param('#/schedule')
                raise JnodeError

            schedule = JsonScheduleArgs.deserialize(schedule_jnode)

        # deserialize #/ticket
        ticket_jnode = rule_jnode.get('ticket')
        if type(rule_jnode) != dict:
            eventlog.log_e_param('#/ticket')
            raise JnodeError

        ticket = JsonTicket.deserialize(ticket_jnode)

        return BaseRuleArgs(name, enabled, ticket, schedule)


class JsonVideoArgs(VideoArgs):
    def __init__(self, *args, **kwargs):
        VideoArgs.__init__(self, *args, **kwargs)

    @staticmethod
    def deserialize(args_jnode):
        """
        :type args_jnode: dict
        :rtype: JsonVideoArgs
        :raise JnodeError:
        """
        pre_record = args_jnode.get('pre-record', 5000)
        post_record = args_jnode.get('post-record', 5000)

        max_w = args_jnode.get('max-width', 32752)
        max_h = args_jnode.get('max-height', 32752)

        quality = args_jnode.get('quality', 500)
        video_format = args_jnode.get('format')

        return JsonVideoArgs(max_w=max_w, max_h=max_h, video_format=video_format, quality=quality,
                             pre_record=pre_record, post_record=post_record)
