from edge.service.tz import ITzService
from .types import ScheduleArgs
from datetime import time


class Schedule:
    def __init__(self, conf, tz_service):
        """
        :type conf: ScheduleArgs | None
        :type tz_service: ITzService
        """
        self._disabled = conf is None
        if self._disabled:
            return

        self._tz_service = tz_service
        self._day_to_limits_map = {
            0: False,
            1: False,
            2: False,
            3: False,
            4: False,
            5: False,
            6: False
        }

        time00 = time(hour=0, minute=0, second=0, microsecond=0)

        if conf.min_time == conf.max_time == time00:
            for day_index in range(0, len(conf.days_to_bool)):
                day_limits = conf.days_to_bool[day_index]
                self._day_to_limits_map[day_index] = day_limits

        elif conf.min_time < conf.max_time:
            for day_index in range(len(conf.days_to_bool)):
                if conf.days_to_bool[day_index]:
                    self._day_to_limits_map[day_index] = [(conf.min_time, conf.max_time)]
                else:
                    self._day_to_limits_map[day_index] = False

        elif conf.min_time > conf.max_time:
            time24 = time(hour=23, minute=59, second=59, microsecond=9999)
            for day_index in range(len(conf.days_to_bool)):
                day_limits = conf.days_to_bool[day_index]
                next_day_index = (day_index + 1) % 7
                if day_limits:
                    if type(self._day_to_limits_map[day_index]) == list:
                        self._day_to_limits_map[day_index].append((conf.min_time, time24))
                    else:
                        self._day_to_limits_map[day_index] = [(conf.min_time, time24)]

                    self._day_to_limits_map[next_day_index] = [(time00, conf.max_time)]

                else:
                    self._day_to_limits_map[day_index] = False

    def check(self, sec_ts):
        """
        :type sec_ts: int
        :rtype: bool
        """
        if self._disabled:
            return True

        dt = self._tz_service.get_datetime(sec_ts)

        day_time_ranges = self._day_to_limits_map.get(dt.weekday())

        if type(day_time_ranges) == bool:
            return day_time_ranges

        elif type(day_time_ranges) == list:
            for day_time_range in day_time_ranges:
                if day_time_range[0] < dt.time() < day_time_range[1]:
                    return True

        return False

    def check2(self, usec_ts):
        """
        :type usec_ts: int
        :rtype: bool
        """
        return self.check(int(usec_ts / 1e6))
