import datetime


class ScheduleArgs:
    def __init__(self, min_time, max_time, days):
        """
        :type min_time: datetime.time
        :type max_time: datetime.time
        :type days: list of bool
        """
        self.min_time = min_time
        self.max_time = max_time
        self.days_to_bool = days

    def serialize(self):
        """
        :rtype: str
        """
        jnode = {
            "min-time": [
                self.min_time.hour,
                self.min_time.minute
            ],
            "max-time": [
                self.max_time.hour,
                self.max_time.minute
            ],
            "days": {
                "mo": self.days_to_bool[0],
                "tu": self.days_to_bool[1],
                "we": self.days_to_bool[2],
                "th": self.days_to_bool[3],
                "fr": self.days_to_bool[4],
                "sa": self.days_to_bool[5],
                "su": self.days_to_bool[6]
            }
        }

        return jnode


class BaseRuleArgs:
    def __init__(self, name, enabled, ticket, schedule=None):
        """
        :type name: str
        :type enabled: bool
        :type ticket: Ticket
        :type schedule: ScheduleArgs | None
        """
        self.name = name
        self.enabled = enabled
        self.ticket = ticket
        self.schedule = schedule


class VideoArgs:
    def __init__(self, max_w=32752, max_h=32752, video_format='h264-mp4', quality=500,
                 pre_record=5000, post_record=5000):
        """
        :type max_w: int
        :type max_h: int
        :type video_format: str
        :type quality: int
        :type pre_record: int
        :type post_record: int
        """

        self.max_w = max_w
        self.max_h = max_h
        self.pre_record = pre_record
        self.post_record = post_record

        self.quality = quality
        self.format = video_format
