
from edge import eventlog
from edge.image.jnode.types import JsonImEngineArgs
from edge.types import JnodeError

from ...com.jnode.types import JsonBaseRuleArgs
from ..types import DetectorArgs, RuleArgs, RuleLimits

from ...types import IDetector
from ....model.types import Identity
from ....model.jnode.types import JsonSceneArgs


class JsonRuleLimits(RuleLimits):
    def __init__(self, *args, **kwargs):
        RuleLimits.__init__(self, *args, **kwargs)

    @staticmethod
    def deserialize(limits_jnode):
        """
        :type limits_jnode: dict
        :rtype: JsonRuleLimits
        :raise JnodeError:
        """
        # deserialize #/max-standing-time
        max_standing_time = limits_jnode.get('max-standing-time')
        if not (int == type(max_standing_time) and max_standing_time > 0):
            eventlog.log_e_param('#/max-standing-time')
            raise JnodeError

        return JsonRuleLimits(max_standing_time)


class JsonRuleArgs(RuleArgs):
    def __init__(self, *args, **kwargs):
        RuleArgs.__init__(self, *args, **kwargs)

    @staticmethod
    def deserialize(rule_jnode):
        """
        :type rule_jnode: dict
        :rtype: JsonRuleArgs
        :raise JnodeError:
        """
        if type(rule_jnode) != dict:
            eventlog.log_e_param('#/')
            raise JnodeError

        # deserialize #/name
        # deserialize #/enabled
        # deserialize #/schedule
        # deserialize #/ticket
        rule = JsonBaseRuleArgs.deserialize(rule_jnode)

        # deserialize #/limits
        limits_jnode = rule_jnode.get('limits')
        if type(rule_jnode) != dict:
            eventlog.log_e_param('#/limits')
            raise JnodeError

        limits = JsonRuleLimits.deserialize(limits_jnode)

        return JsonRuleArgs(
            rule.name, rule.enabled, rule.schedule, rule.ticket,
            limits)


class JsonDetectorArgs(DetectorArgs):
    def __init__(self, *args, **kwargs):
        DetectorArgs.__init__(self, *args, **kwargs)

    @staticmethod
    def deserialize(jnode, index, def_footer_mapping):
        """
        :type jnode: dict
        :type index: int
        :type def_footer_mapping: dict
        :rtype: JsonDetectorArgs
        :raise JnodeError:
        """
        # deserialize #/name
        name = jnode.get('name')
        if not type(name) in [str, unicode] or len(name) > 32:
            eventlog.log_e_param('#/name')
            raise JnodeError

        # deserialize #/store
        store = jnode.get('store')
        if not (type(store) in [str, unicode] and 1 <= len(store) <= 2):
            eventlog.log_e_param('#/store')
            raise JnodeError

        # deserialize #/scene
        scene_jnode = jnode.get('scene')
        if type(scene_jnode) != dict:
            eventlog.log_e_param('#/scene')
            raise JnodeError
        scene = JsonSceneArgs.deserialize(scene_jnode)

        # deserialize #/rules
        rules_jnode = jnode.get('rules')
        if type(rules_jnode) != list:
            eventlog.log_e_param('#/rules')
            raise JnodeError

        rules = filter(lambda x: x.enabled, map(JsonRuleArgs.deserialize, rules_jnode))
        identity = Identity(IDetector.TYPE_PARKING, index)

        # deserialize #/images
        images_jnode = jnode.get('images')
        if type(images_jnode) != dict:
            eventlog.log_e_param('#/images')
            raise JnodeError

        init_im_engine = JsonImEngineArgs.deserialize(images_jnode['violation'])
        fini_im_engine = init_im_engine

        # deserialize range tout
        range_tout = jnode.get('range-tout')
        if type(range_tout) not in [int, long]:
            eventlog.log_e_param('#/range_tout')
            raise JnodeError

        # deserialize lost tout
        lost_tout = jnode.get('lost-tout')
        if type(lost_tout) not in [int, long]:
            eventlog.log_e_param('#/lost_tout')
            raise JnodeError

        return JsonDetectorArgs(
            identity, name, store, scene, rules,
            init_im_engine, fini_im_engine, def_footer_mapping, range_tout, lost_tout)
