from edge.image.types import ImEngineArgs
from edge.service.tz import ITzService
from edge.model.types import Origin

from ..com.types import ScheduleArgs
from ...model.types import Identity, Evidence, SceneArgs


class RuleLimits:
    def __init__(self, max_standing_time):
        """
        :type max_standing_time: int
        """
        self.max_standing_time = max_standing_time


class RuleArgs:
    def __init__(self, name, enabled, schedule, ticket, limits):
        """
        :type name: str
        :type enabled: bool
        :type schedule: ScheduleArgs | None
        :type ticket: Ticket
        :type limits: RuleLimits
        """
        self.name = name
        self.enabled = enabled
        self.schedule = schedule
        self.ticket = ticket
        self.limits = limits


class DetectorArgs:
    def __init__(self, identity, name, store, scene, rules, init_im_engine, fini_im_engine, def_footer_mapping,
                 range_tout, lost_tout):
        """
        :type identity: Identity
        :type name: str
        :type store: str
        :type scene: SceneArgs
        :type rules: list[RuleArgs]
        :type init_im_engine: ImEngineArgs
        :type fini_im_engine: ImEngineArgs
        :type def_footer_mapping: dict
        :type range_tout: long
        :type lost_tout: long
        """
        self.identity = identity
        self.name = name
        self.store = store
        self.scene = scene
        self.rules = rules
        self.init_im_engine = init_im_engine
        self.fini_im_engine = fini_im_engine
        self.def_footer_mapping = def_footer_mapping
        self.range_tout = range_tout * 1e6
        self.lost_tout = lost_tout * 1e6


class ParkingEvidence(Evidence):
    def __init__(self, detector_name, rule_args, ts, origin, standing_time):
        """
        :type detector_name: str
        :type rule_args: RuleArgs
        :type ts: long
        :type origin: Origin
        :type standing_time: long
        """
        self._detector_name = detector_name
        self._standing_time = standing_time
        self.rule_args = rule_args

        Evidence.__init__(self, Evidence.NAME_PARKING, origin, ts)

    @property
    def standing_time(self):
        """
        :rtype: long
        """
        return self._standing_time

    def serialize(self, tz_service):
        """
        :type tz_service: ITzService
        :rtype: dict
        """
        jnode = Evidence.serialize(self, tz_service)
        waypoints_jnode = map(lambda x: x.serialize(tz_service), self._waypoints)
        jnode['evidence'] = {
            'detector': {
                'name': self._detector_name,
                'rule': {
                    'name': self.rule_args.name,
                    'schedule': self.rule_args.schedule.serialize(),
                    'ticket': {
                        'name': self.rule_args.ticket.name
                    },
                    'limits': {
                        'max-standing-time': self.rule_args.limits.max_standing_time
                    }
                }
            },
            'measurements': {
                'standing-time': self._standing_time,
                'waypoints': {
                    'violation': waypoints_jnode
                }
            }
        }

        return jnode
