from edge.image.types import ImEngineArgs
from edge.service.tz import ITzService
from edge.model.types import Origin

from ...model.types import Identity, Evidence, SceneArgs, RedLightViolationWaypoint
from ..com.types import VideoArgs


class RuleArgs:
    def __init__(self, name, ticket):
        """
        :type name: str
        :type ticket: Ticket
        """
        self.name = name
        self.ticket = ticket


class DetectorArgs:
    def __init__(self, identity, name, store, scene, rules, im_engine, def_footer_mapping, video, direction):
        """
        :type identity: Identity
        :type name: str
        :type store: str
        :type scene: SceneArgs
        :type rules: list of RuleArgs
        :type im_engine: ImEngineArgs
        :type def_footer_mapping: dict
        :type video: VideoArgs | None
        :type direction: str
        """
        self.identity = identity
        self.name = name
        self.store = store
        self.scene = scene
        self.rules = rules
        self.im_engine = im_engine
        self.def_footer_mapping = def_footer_mapping
        self.video = video
        self.receding_direction = direction == 'receding'


class RedLightEvidence(Evidence):
    def __init__(self, origin, ts, args, detector_name):
        """
        :type origin: Origin
        :type ts: long
        :type args: RuleArgs
        :type detector_name: str
        """
        Evidence.__init__(self, Evidence.NAME_RED_LIGHT, origin, ts)
        self.detector_name = detector_name
        self.args = args

    def serialize(self, tz_service):
        """
        :type tz_service: ITzService
        :rtype: dict
        """
        jnode = Evidence.serialize(self, tz_service)
        jnode['evidence'] = {

        }
        # waypoints_jnode = dict()
        # for waypoint in self.waypoints:
        #     waypoints_jnode[waypoint.name] = waypoint.serialize(tz_service)

        pre_stopbar_waypoints = filter(lambda waypoint: waypoint.name == waypoint.NAME_PRE_STOPBAR, self._waypoints)
        post_stopbar_waypoints = filter(lambda waypoint: waypoint.name == waypoint.NAME_POST_STOPBAR, self._waypoints)

        pre_waypoints_jnode = map(lambda x: x.serialize(tz_service), pre_stopbar_waypoints)
        post_waypoints_jnode = map(lambda x: x.serialize(tz_service), post_stopbar_waypoints)

        jnode['evidence'] = {
            'detector': {
                'name': self.detector_name,
                'rule': {
                    'name': self.args.name,
                    'ticket': {
                        'name': self.args.ticket.name
                    }
                }
            },
            'measurements': {
                'waypoints': {
                    'pre-stopbar': pre_waypoints_jnode,
                    'post-stopbar': post_waypoints_jnode
                }
            }
        }

        if len(self._videos):
            videos_jnode = map(lambda x: x.serialize(tz_service), self._videos)
            jnode['evidence']['measurements']['videos'] = videos_jnode

        return jnode

    def get_ukr_json_ev(self):
        """
        :return (green_amber_time,red_time, red_amber_time)
        :rtype: (int, int,  int)
        """
        post_stopbar_waypoints = filter(lambda waypoint: waypoint.name == waypoint.NAME_POST_STOPBAR, self._waypoints)
        post_stopbar_waypoints.sort(key=lambda item: item.ts, reverse=True)

        last_post_stopbar_wp = post_stopbar_waypoints[0]  # type:RedLightViolationWaypoint

        green_amder_t = last_post_stopbar_wp.green_amber_time
        red_t = last_post_stopbar_wp.red_time
        red_amber_t = last_post_stopbar_wp.red_amber_time

        return green_amder_t, red_t, red_amber_t
