
from ..model.models import Track


class DetectorError(RuntimeError):
    pass


class IBaseDetector:
    def __init__(self):
        pass

    def set_handler(self, event_handler):
        """
        :type event_handler: (Track, BaseEvent) -> None
        """
        raise NotImplementedError

    def analyze(self, track):
        """
        :type track: Track
        """
        raise NotImplementedError

    def destroy(self):
        pass

    def collect_stats(self, json_handler):
        """
        :type json_handler: (dict) -> None
        :rtype: None
        """
        pass


class IDetector:
    TYPE_TRANSIT = 0
    TYPE_PARKING = 1
    TYPE_WRONG_WAY = 2
    TYPE_SPOT_SPEED = 3
    TYPE_RED_LIGHT = 4
    TYPE_WANTED = 5
    TYPE_SECTION_SPEED = 6
    TYPE_BUS_LANE = 7

    def __init__(self):
        pass

    def set_handler(self, event_handler):
        """
        :type event_handler: (Event) -> None
        """
        raise NotImplementedError

    def analyze(self, track, base):
        """
        :type track: Track
        :type base: BaseEvent
        """
        raise NotImplementedError

    def collect_stats(self, json_handler):
        """
        :type json_handler: (dict) -> None
        :rtype: None
        """
        pass

    def destroy(self):
        pass
