from edge.image.types import MemBlob
from edge.service.tz import ITzService
from edge.model.types import Origin
from edge.service import db_wanted

from ...model.types import Identity, Evidence


class LimitsArgs:
    def __init__(self, plate_dist, list_owner, list_name):
        """
        :type plate_dist: int
        :type list_owner: str | None
        :type list_name: str | None
        """
        self.plate_dist = plate_dist
        self.list_owner = list_owner
        self.list_name = list_name


class RuleArgs:
    def __init__(self, name, enabled, ticket, limits):
        """
        :type name: str
        :type enabled: bool
        :type ticket: Ticket
        :type limits: LimitsArgs
        """
        self.name = name
        self.enabled = enabled
        self.ticket = ticket
        self.limits = limits


class DetectorArgs:
    def __init__(self, identity, name, store, rules):
        """
        :type name: str
        :type store: str
        :type identity: Identity
        :type rules: list[RuleArgs]
        """
        self.name = name
        self.store = store
        self.identity = identity
        self.rules = rules


class WantedEvidence(Evidence):

    def __init__(self, origin, ts, detector_name, args, matches):
        """
        :type origin: Origin
        :type ts: long
        :type detector_name: str
        :type args: RuleArgs
        :type matches: list[db_wanted.ListMatch]
        """
        self._matches = matches
        self._detector_name = detector_name
        self._args = args
        Evidence.__init__(self, Evidence.NAME_WANTED, origin, ts)

    def get_blobs(self):
        """
        :rtype: list of MemBlob
        """
        return []

    def serialize(self, tz_service):
        """
        :type tz_service: ITzService
        :rtype: dict
        """
        jnode = Evidence.serialize(self, tz_service)
        jnode['evidence'] = {
            'detector': {
                'name': self._detector_name,
                'rule': {
                    'name': self._args.name,
                    'ticket': {
                        'name': self._args.ticket.name
                    },
                    'limits': {
                        'plate-dist': self._args.limits.plate_dist,
                        'plate-list': {
                            'owner': self._args.limits.list_owner,
                            'name': self._args.limits.list_name
                        }
                    }
                }
            },
            'measurements': {
                'matches': map(lambda x: x.serialize(), self._matches)
            }
        }

        return jnode
