from itertools import izip

from edge import eventlog
from edge_bbox.types import RectF
from edge.types import JnodeError
from ..types import Ticket, Mask, SceneArgs


class JsonSceneArgs(SceneArgs):
    def __init__(self, *args, **kwargs):
        SceneArgs.__init__(self, *args, **kwargs)

    @staticmethod
    def deserialize(zone_jnode):
        """
        :type zone_jnode: dict
        :rtype: JsonSceneArgs
        :raise JnodeError:
        """
        area_jnode = zone_jnode.get('area')
        if type(area_jnode) != dict:
            eventlog.log_e_param('#/area')
            raise JnodeError

        # deserialize #/area/mask
        mask = JsonMask.deserialize(area_jnode.get('mask'))

        # deserialize #/area/scene-wh
        scene_wh_jnode = area_jnode.get('scene-wh')
        if not type(scene_wh_jnode) in [list, tuple] or len(scene_wh_jnode) != 2:
            eventlog.log_e_param('#/area/scene-wh')
            raise JnodeError

        w, h = scene_wh_jnode
        if not (int == type(w) and 640 <= w <= 65535):
            eventlog.log_e_param('#area/scene-wh/0')
            raise JnodeError

        if not (int == type(h) and 480 <= h <= 65535):
            eventlog.log_e_param('#area/scene-wh/1')
            raise JnodeError

        return JsonSceneArgs(mask, (w, h))


class JsonTicket(Ticket):
    def __init__(self, *args, **kwargs):
        Ticket.__init__(self, *args, **kwargs)

    @staticmethod
    def deserialize(ticket_jnode):
        """
        :type ticket_jnode: dict
        :rtype: Ticket
        :raise JnodeError:
        """
        # deserialize #/name
        name = ticket_jnode.get('name')
        if not (type(name) in [str, unicode]) or len(name) > 128:
            eventlog.log_e_param('#/name')
            raise JnodeError

        return Ticket(name)


class JsonMask(Mask):
    def __init__(self, *args, **kwargs):
        Mask.__init__(self, *args, **kwargs)

    @staticmethod
    def deserialize(mask_jnode):
        """
        :type mask_jnode: list[(int, int)]
        :rtype: Mask
        :raise JnodeError:
        """
        if type(mask_jnode) != list:
            eventlog.log_e_param('#/')
            raise JnodeError

        index = 0
        while index < len(mask_jnode):
            point_jnode = mask_jnode[index]
            if not (type(point_jnode) in [list, tuple] or len(point_jnode) != 2):
                eventlog.log_e_param('#/%s' % index)
                raise JnodeError

            x, y = point_jnode
            if not (int == type(x) and 0 <= x <= 65535):
                eventlog.log_e_param('#/%s/0' % index)
                raise JnodeError

            if not (int == type(y) and 0 <= y <= 65535):
                eventlog.log_e_param('#/%s/1' % index)
                raise JnodeError

            index += 1

        return JsonMask(mask_jnode)

    @staticmethod
    def deserialize_list(mask_jnode):
        """
        :type mask_jnode: list[int]
        :rtype: Mask
        :raise JnodeError:
        """
        if type(mask_jnode) != list:
            eventlog.log_e_param('#/')
            raise JnodeError

        it = iter(mask_jnode)
        mask_jnode = list(izip(it, it))

        return JsonMask.deserialize(mask_jnode)


class JsonRectF(RectF):
    def __init__(self, *args, **kwargs):
        RectF.__init__(self, *args, **kwargs)

    @staticmethod
    def deserialize(jnode):
        """
        :type jnode: list of float
        :rtype: JsonRectF
        """
        if len(jnode) != 4:
            raise JnodeError

        return JsonRectF(*jnode)
