from types import PlateInfoServiceConf
from edge.com import fs
from edge import eventlog


class PlateTemplateError(RuntimeError):
    pass


class IPlateInfoService:
    def __init__(self):
        pass

    def get_country_code(self, template_ref):
        """
        :type template_ref: long
        :rtype: int
        """
        raise NotImplementedError

    def get_plate_attrs(self, template_ref):
        """
        :type template_ref: long
        :rtype: str
        """
        raise NotImplementedError


class _PlateInfoService(IPlateInfoService):
    def __init__(self, conf):
        """
        :type conf: PlateInfoServiceConf
        """
        IPlateInfoService.__init__(self)
        if conf.info_path:
            self._ref_to_attrs_map = fs.load_json3(conf.info_path)
        else:
            self._ref_to_attrs_map = dict()

    def get_country_code(self, template_ref):
        """
        :type template_ref: long
        :rtype: int
        """
        return (template_ref >> 20) & 0xFFF

    def get_plate_attrs(self, template_ref):
        """
        :type template_ref: long
        :rtype: str | None
        """
        plate_attrs = self._ref_to_attrs_map.get(str(template_ref)) or None
        if plate_attrs is None:
            eventlog.log_w('fail: unsupported (template-ref:{})'.format(template_ref))

        return plate_attrs


def init_service(conf):
    """
    :type conf: PlateInfoServiceConf
    :rtype: IPlateInfoService
    """
    return _PlateInfoService(conf)


# noinspection PyUnusedLocal
def fini_service(service):
    """
    :type service: IPlateInfoService | _PlateInfoService
    """
    pass
