
from .types import SystestServiceArgs
from edge.com import fs


class ISystestService:
    def __init__(self):
        pass

    def serialize(self, jnode):
        """
        :type jnode: dict
        :rtype: dict
        """
        raise NotImplementedError


def init_service(args):
    """
    :type args: SystestServiceArgs
    :rtype: ISystestService
    """
    if args.conf_path:
        systest = _SystestService(args)
    else:
        systest = _StubSystestService()

    return systest


# noinspection PyUnusedLocal
def fini_service(service):
    """
    :type service: ISystestService | _SystestService
    """
    pass


class _StubSystestService(ISystestService):
    def __init__(self):
        ISystestService.__init__(self)
        pass

    def serialize(self, jnode):
        """
        :type jnode: dict
        :rtype: dict
        """
        return jnode


class _SystestService(ISystestService):
    def __init__(self, args):
        """
        :type args: SystestServiceArgs
        """
        ISystestService.__init__(self)

        jnode = fs.load_json3(args.conf_path, default=dict())
        rw_jnode = jnode.get('rw') or dict()
        self._crt_jnode = rw_jnode.get('certificate')

    def serialize(self, jnode):
        """
        :type jnode: dict
        :rtype: dict
        """
        if self._crt_jnode:
            jnode.update({'certificate': self._crt_jnode})

        return jnode

