import collections
from _models import TrSceneItem


class LrRaMatchStore:
    """
    :type _lr_source: (int, int) | None
    """
    def __init__(self, twisted_reactor, match_tout_sec):
        self._twisted_reactor = twisted_reactor
        self._match_tout_sec = match_tout_sec

        self._lr_oid_to_delayed_call_map = dict()
        self._lr_oid_to_ra_oid_to_match_count_map = collections.defaultdict(collections.Counter)

        self._lr_source = None

    def _handle_lr_oid_tout(self, lr_oid):
        self._lr_oid_to_ra_oid_to_match_count_map.pop(lr_oid, None)
        self._lr_oid_to_delayed_call_map.pop(lr_oid, None)

    def add(self, lr_source, tr_scene_tr_items):
        """
        :type lr_source: (int, int)
        :type tr_scene_tr_items: list[TrSceneItem]
        """

        if self._lr_source == lr_source:
            for tr_item in tr_scene_tr_items:
                # new lr_oid will be inserted.
                lr_oid = tr_item.lr_item.oid
                rm_delayed_call = self._lr_oid_to_delayed_call_map.get(lr_oid, None)
                if rm_delayed_call:
                    rm_delayed_call.reset(self._match_tout_sec)

                else:
                    rm_delayed_call = self._twisted_reactor.callLater(
                        self._match_tout_sec, self._handle_lr_oid_tout, lr_oid)
                    self._lr_oid_to_delayed_call_map[lr_oid] = rm_delayed_call

                self._lr_oid_to_ra_oid_to_match_count_map[lr_oid][tr_item.ra_item.oid] += 1

        else:
            self._lr_oid_to_ra_oid_to_match_count_map.clear()
            self._lr_oid_to_delayed_call_map.clear()
            self._lr_source = lr_source

    def get_ra_oids(self, lr_source, lr_track_oids):
        """
        :type lr_source: (int, int)
        :type lr_track_oids: list[int]
        :rtype: list[int]
        """
        if self._lr_source != lr_source:
            return []

        final_ra_oid_to_match_count_map = collections.Counter()
        for lr_oid in lr_track_oids:
            ra_oid_to_match_count_map = self._lr_oid_to_ra_oid_to_match_count_map.get(lr_oid, None)
            if ra_oid_to_match_count_map:
                final_ra_oid_to_match_count_map += ra_oid_to_match_count_map

        # sort entries by count in desc order
        ra_oids_desc = sorted(final_ra_oid_to_match_count_map,
                              key=final_ra_oid_to_match_count_map.get,
                              reverse=True)

        return ra_oids_desc


class RaLrMatchStore:
    """
    :type _lr_source: (int, int) | None
    """
    def __init__(self, twisted_reactor, match_tout_sec):
        self._twisted_reactor = twisted_reactor
        self._match_tout_sec = match_tout_sec

        self._lr_oid_to_delayed_call_map = dict()
        self._lr_oid_to_ra_oid_to_match_count_map = collections.defaultdict(collections.Counter)

        self._lr_source = None

    def _handle_lr_oid_tout(self, lr_oid):
        self._lr_oid_to_ra_oid_to_match_count_map.pop(lr_oid, None)
        self._lr_oid_to_delayed_call_map.pop(lr_oid, None)

    def add(self, lr_source, tr_scene_tr_items):
        """
        :type lr_source: (int, int)
        :type tr_scene_tr_items: list[TrSceneItem]
        """

        if self._lr_source == lr_source:
            for tr_item in tr_scene_tr_items:
                # new lr_oid will be inserted.
                lr_oid = tr_item.lr_item.oid
                rm_delayed_call = self._lr_oid_to_delayed_call_map.get(lr_oid, None)
                if rm_delayed_call:
                    rm_delayed_call.reset(self._match_tout_sec)

                else:
                    rm_delayed_call = self._twisted_reactor.callLater(
                        self._match_tout_sec, self._handle_lr_oid_tout, lr_oid)
                    self._lr_oid_to_delayed_call_map[lr_oid] = rm_delayed_call

                self._lr_oid_to_ra_oid_to_match_count_map[lr_oid][tr_item.ra_item.oid] += 1

        else:
            self._lr_oid_to_ra_oid_to_match_count_map.clear()
            self._lr_oid_to_delayed_call_map.clear()
            self._lr_source = lr_source

    def get_ra_oids(self, lr_source, lr_track_oids):
        """
        :type lr_source: (int, int)
        :type lr_track_oids: list[int]
        :rtype: list[int]
        """
        if self._lr_source != lr_source:
            return []

        final_ra_oid_to_match_count_map = collections.Counter()
        for lr_oid in lr_track_oids:
            ra_oid_to_match_count_map = self._lr_oid_to_ra_oid_to_match_count_map.get(lr_oid, None)
            if ra_oid_to_match_count_map:
                final_ra_oid_to_match_count_map += ra_oid_to_match_count_map

        # sort entries by count in desc order
        ra_oids_desc = sorted(final_ra_oid_to_match_count_map,
                              key=final_ra_oid_to_match_count_map.get,
                              reverse=True)

        return ra_oids_desc
