from os import rename
from time import time, gmtime, strftime

from edge import eventlog
from .._models import TrTrack


class TrackReactCounters:
    def __init__(self):
        self.track_count = long(0)

    def inc_track_count(self):
        self.track_count += 1


class SceneReactCounters:
    def __init__(self):
        self.scene_count = long(0)
        self.lr_scene_count = long(0)
        self.ra_scene_count = long(0)
        self.lr_scene_item_count = long(0)
        self.ra_scene_item_count = long(0)

    def inc_scene_count(self):
        self.scene_count += 1

    def inc_lr_scene_item_count(self, lr_scene_item_count):
        self.lr_scene_count += 1
        self.lr_scene_item_count += lr_scene_item_count

    def inc_ra_scene_item_count(self, ra_scene_item_count):
        self.ra_scene_count += 1
        self.ra_scene_item_count += ra_scene_item_count


class TrackEventCounters:
    def __init__(self):
        self.track_count = long(0)

        self.lrra_track_count = long(0)
        self.lr_track_count = long(0)
        self.ra_track_count = long(0)

        self.lr_track_dublicate_count = long(0)

        self.lrra_track_time = long(0)
        self.lr_track_time = long(0)
        self.ra_track_time = long(0)

    def inc_dublicate_count(self):
        self.lr_track_dublicate_count += 1

    def inc_track_count(self, tr_track):
        """
        :type tr_track: TrTrack
        """
        self.track_count += 1

        if bool(tr_track.lr_track) and bool(tr_track.lr_track.src_jnode) and bool(tr_track.ra_track):
            ctime = min(tr_track.lr_track.src_jnode['PlateTrace'][0]['Ts'], tr_track.ra_track.catch_ts)
            ltime = max(tr_track.lr_track.src_jnode['PlateTrace'][-1]['Ts'], tr_track.ra_track.lost_ts)

            self.lrra_track_time += ltime - ctime
            self.lrra_track_count += 1

        if bool(tr_track.lr_track) and bool(tr_track.lr_track.src_jnode):
            self.lr_track_time += tr_track.lr_track.src_jnode['PlateTrace'][-1]['Ts'] -\
                                  tr_track.lr_track.src_jnode['PlateTrace'][0]['Ts']
            self.lr_track_count += 1

        if bool(tr_track.ra_track):
            self.ra_track_time += tr_track.ra_track.lost_ts - tr_track.ra_track.catch_ts
            self.ra_track_count += 1


class SceneEventCounters:
    def __init__(self, scene_consumer_count):
        self.scene_count_list = [long(0)] * scene_consumer_count

    def inc_scene_count(self, consumer_index):
        self.scene_count_list[consumer_index] += 1


class StatCounters:
    def __init__(self, twisted_reactor, scene_consumer_count, stat_filepath, stat_tout_sec):
        self.track_react = TrackReactCounters()
        self.scene_react = SceneReactCounters()

        self.track_event = TrackEventCounters()
        self.scene_event = SceneEventCounters(scene_consumer_count)

        if stat_tout_sec:
            self._report_stat_counters(twisted_reactor, stat_filepath, stat_tout_sec)

    def _report_stat_counters(self, twisted_reactor, stat_filepath, stat_tout_sec):
        sys_utc_time = time()
        stat_file_lines = [
            # header
            'stat.time-str: %s\n' % strftime('%d.%m.%Y %H:%M:%S', gmtime(sys_utc_time)),
            'stat.time-sec: %s\n' % int(round(sys_utc_time)),
            'stat.tout-sec: %s\n\n' % stat_tout_sec,

            # react-counters
            'react.track-count: %s\n\n' % self.track_react.track_count,

            'react.scene-count: %s\n' % self.scene_react.scene_count,
            'react.scene.lr-scene-count: %s\n' % self.scene_react.lr_scene_count,
            'react.scene.ra-scene-count: %s\n' % self.scene_react.ra_scene_count,
            'react.scene.lr-scene-item-count: %s\n' % self.scene_react.lr_scene_item_count,
            'react.scene.ra-scene-item-count: %s\n\n' % self.scene_react.ra_scene_item_count,

            # event-counters
            'event.track-count: %s\n' % self.track_event.track_count,
            'event.track.lr-track-count: %s\n' % self.track_event.lr_track_count,
            'event.track.lr-track-dup-count: %s\n' % self.track_event.lr_track_dublicate_count,
            'event.track.ra-track-count: %s\n' % self.track_event.ra_track_count,
            'event.track.lrra-track-count: %s\n' % self.track_event.lrra_track_count,

            # time-counters
            'event.track.lr-track-time: %.3f\n' % float(self.track_event.lr_track_time / 1000000.0),
            'event.track.ra-track-time: %.3f\n' % float(self.track_event.ra_track_time / 1000000.0),
            'event.track.lrra-track-time: %.3f\n' % float(self.track_event.lrra_track_time / 1000000.0)
        ]

        eventlog.log_d('tx-scene-count-list: %s' % self.scene_event.scene_count_list)

        # noinspection PyBroadException
        try:
            tmp_filepath = stat_filepath + '.tmp'
            with open(tmp_filepath, 'w') as tmp_file:
                tmp_file.writelines(stat_file_lines)

            rename(tmp_filepath, stat_filepath)
        except Exception:
            eventlog.log_x()

        twisted_reactor.callLater(
            stat_tout_sec, self._report_stat_counters, twisted_reactor, stat_filepath, stat_tout_sec)
